//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 8
#property indicator_color1  LimeGreen
#property indicator_color2  PaleVioletRed
#property indicator_color3  LimeGreen
#property indicator_color4  PaleVioletRed
#property indicator_color5  LimeGreen
#property indicator_color6  PaleVioletRed
#property indicator_color7  LimeGreen
#property indicator_color8  PaleVioletRed
#property indicator_minimum 0
#property indicator_maximum 5

//
//
//
//
//

extern int    DMI.Length            = 14;
extern int    LinesWidth            = 0;
extern string UniqueID              = "Wilders DMI slopes";
extern color  LabelsColor           = DarkGray;
extern int    LabelsHorizontalShift = 7;
extern double LabelsVerticalShift   = 1.5;

//
//
//
//
//

double slopeDIpu[];
double slopeDIpd[];
double slopeDImu[];
double slopeDImd[];
double slopeADXu[];
double slopeADXd[];
double slopeADXRu[];
double slopeADXRd[];


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   SetIndexBuffer(0,slopeDIpu);
   SetIndexBuffer(1,slopeDIpd);
   SetIndexBuffer(2,slopeDImu);
   SetIndexBuffer(3,slopeDImd);
   SetIndexBuffer(4,slopeADXu);
   SetIndexBuffer(5,slopeADXd);
   SetIndexBuffer(6,slopeADXRu);
   SetIndexBuffer(7,slopeADXRd);
      for (int i=0; i<8; i++) { SetIndexStyle(i,DRAW_ARROW,EMPTY,LinesWidth); SetIndexArrow(i,110); }

   //
   //
   //
   //
   //

   IndicatorShortName(UniqueID);
   return(0);
}
int deinit()
{
   for (int t=0; t<4; t++) ObjectDelete(UniqueID+t);
   return(0); 
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][11];
#define _DIp        0
#define _DIm        1
#define _ADX        2
#define _ADXR       3
#define _averageDIp 4
#define _averageDIm 5
#define _averageTR  6
#define _slopeDIp   7
#define _slopeDIm   8
#define _slopeADX   9
#define _slopeADXR 10

//
//
//
//
//

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);
           if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);

           //
           //
           //
           //
           //
         
           static bool initialized = false;
           if (!initialized)
           {
              initialized = true;
              int window = WindowFind(UniqueID);
              for (int t=0; t<4; t++)
              {
                 switch(t)
                 {
                    case 0: string label = "ADXR"; break;
                    case 1:        label = "ADX";  break;
                    case 2:        label = "DI-";  break;
                    case 3:        label = "DI+";  break;
                 }
                 ObjectCreate(UniqueID+t,OBJ_TEXT,window,0,0);
                    ObjectSet(UniqueID+t,OBJPROP_COLOR,LabelsColor);
                    ObjectSet(UniqueID+t,OBJPROP_PRICE1,t+LabelsVerticalShift);
                    ObjectSetText(UniqueID+t,label,8,"Arial");
              }               
           }
           for (t=0; t<4; t++) ObjectSet(UniqueID+t,OBJPROP_TIME1,Time[0]+Period()*LabelsHorizontalShift*60);

   //
   //
   //
   //
   //
   
   double sf = (DMI.Length-1.0)/DMI.Length;
      for (i=limit, r=Bars-i-1;i>=0;i--,r++)
      {
         double currTR  = MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]);
         double DeltaHi = High[i] - High[i+1];
	      double DeltaLo = Low[i+1] - Low[i];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            work[r][_averageDIp] = sf*work[r-1][_averageDIp] + plusDM;
            work[r][_averageDIm] = sf*work[r-1][_averageDIm] + minusDM;
            work[r][_averageTR]  = sf*work[r-1][_averageTR]  + currTR;

         //
         //
         //
         //
         //
                  
            work[r][_DIp] = 0.00;                   
            work[r][_DIm] = 0.00;                   
            if (work[r][_averageTR] > 0)
               {              
                  work[r][_DIp] = 100.00 * work[r][_averageDIp]/work[r][_averageTR];
                  work[r][_DIm] = 100.00 * work[r][_averageDIm]/work[r][_averageTR];
               }            
               double DX;
               if((work[r][_DIp] + work[r][_DIm])>0) 
                    DX = 100*MathAbs(work[r][_DIp] - work[r][_DIm])/(work[r][_DIp] + work[r][_DIm]); 
               else DX = 0.00;
               work[r][_ADX]       = sf*work[r-1][_ADX] + DX/DMI.Length; 
               work[r][_ADXR]      = 0.5*(work[r][_ADX] + work[r-DMI.Length][_ADX]);
               work[r][_slopeDIp]  = work[r-1][_slopeDIp];
               work[r][_slopeDIm]  = work[r-1][_slopeDIm];
               work[r][_slopeADX]  = work[r-1][_slopeADX];
               work[r][_slopeADXR] = work[r-1][_slopeADXR];
                  if (work[r][_DIp]  > work[r-1][_DIp])  work[r][_slopeDIp]  =  1;
                  if (work[r][_DIp]  < work[r-1][_DIp])  work[r][_slopeDIp]  = -1;
                  if (work[r][_DIm]  > work[r-1][_DIm])  work[r][_slopeDIm]  =  1;
                  if (work[r][_DIm]  < work[r-1][_DIm])  work[r][_slopeDIm]  = -1;
                  if (work[r][_ADX]  > work[r-1][_ADX])  work[r][_slopeADX]  =  1;
                  if (work[r][_ADX]  < work[r-1][_ADX])  work[r][_slopeADX]  = -1;
                  if (work[r][_ADXR] > work[r-1][_ADXR]) work[r][_slopeADXR] =  1;
                  if (work[r][_ADXR] < work[r-1][_ADXR]) work[r][_slopeADXR] = -1;

            slopeDIpu[i]  = EMPTY_VALUE; if (work[r][_slopeDIp] == 1) slopeDIpu[i]  = 4;
            slopeDIpd[i]  = EMPTY_VALUE; if (work[r][_slopeDIp] ==-1) slopeDIpd[i]  = 4;
            slopeDImu[i]  = EMPTY_VALUE; if (work[r][_slopeDIm] == 1) slopeDImu[i]  = 3;
            slopeDImd[i]  = EMPTY_VALUE; if (work[r][_slopeDIm] ==-1) slopeDImd[i]  = 3;
            slopeADXu[i]  = EMPTY_VALUE; if (work[r][_slopeADX] == 1) slopeADXu[i]  = 2;
            slopeADXd[i]  = EMPTY_VALUE; if (work[r][_slopeADX] ==-1) slopeADXd[i]  = 2;
            slopeADXRu[i] = EMPTY_VALUE; if (work[r][_slopeADXR]== 1) slopeADXRu[i] = 1;
            slopeADXRd[i] = EMPTY_VALUE; if (work[r][_slopeADXR]==-1) slopeADXRd[i] = 1;
      }   
   return(0);      
}